require("HSK")

--- run tests
function HSK:Test()
  HSK:Debug("Starting SK List tests")
  local tmp, tmpN, tmpTable
  tmpN = HSK:GetMasterListSize()
  assert(0 == tmpN, "The Master list should still be empty, but is "..(tmpN or "nil"))
  tmpN = HSK:GetPresentListSize()
  assert(0 == tmpN, "The Present list should still be empty, but is "..(tmpN or "nil"))

  -- SetRoles
  HSK:Print("*** Testing SetRoles()")
  assert(not HSK:SetRoles(), "SetRoles should fail without a role table")
  tmpN = HSK:SetRoles(Test.Roles)
  assert(tmpN == (#Test.Roles), "SetRoles should return "..(#Test.Roles).." roles in set, got: "..(tmpN or "nil"))

  -- AddMember
  HSK:Print("*** Testing AddMember()")
  assert(not HSK:AddMember(), "AddMember should return nil with no arguments")
  assert(not HSK:AddMember(Test.Mains[1]), "AddMember should return nil with no role argument")
  assert(not HSK:AddMember(Test.Mains[1],Test.BadRole), "AddMember should return nil with bad role argument")
  assert(not HSK:AddMember(Test.Mains[1],Test.Roles[1],0), "AddMember should return nil with position <= 0")
  assert(not HSK:AddMember(Test.Mains[1],Test.Roles[1],-10), "AddMember should return nil with position <= 0")
  tmpN = HSK:GetMasterListSize()
  assert(0 == tmpN, "The Master list should still be empty, but is "..(tmpN or "nil"))
  tmpN = HSK:GetPresentListSize()
  assert(0 == tmpN, "The Present list should still be empty, but is "..(tmpN or "nil"))

  tmp = HSK:AddMember(Test.Mains[1],Test.Roles[1])
  assert(tmp == 1, "The first member should be in position 1, but is "..(tmp and "true" or "nil"))
  tmp = HSK:GetRole(Test.Mains[1])
  assert(tmp == Test.Roles[1], Test.Mains[1].." should have raid role \""..Test.Roles[1].."\", but has "..(tmp and "true" or "nil"))
  tmpN = HSK:GetRoleCount(Test.Roles[1])
  assert(tmpN == 1, "Should have 1 member with role \""..Test.Roles[1].."\" but have "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[1])
  assert(tmpN == 1, Test.Mains[1].." should have 1 toon, but has "..(tmpN or "nil"))

  tmp = HSK:AddMember(Test.Mains[1],Test.Roles[1])
  assert(tmp == 1, "The first member should still be in position 1, but is "..(tmp and "true" or "nil"))
  assert(HSK:GetMasterListSize() == 1, "The Master list should still have only 1 member, but has "..(tmp and "true" or "nil"))
  tmpN = HSK:GetRoleCount(Test.Roles[1])
  assert(tmpN == 1, "Should have 1 member with role \""..Test.Roles[1].."\" but have "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[1])
  assert(tmpN == 1, Test.Mains[1].." should have 1 toon, but has "..(tmpN or "nil"))

  tmp = HSK:AddMember(Test.Mains[2],Test.Roles[2],3)
  assert(tmp == 2, "The second member should be in position 2, but is "..(tmp and "true" or "nil"))
  assert(HSK:GetMasterListSize() == 2, "The Master list should now have 2 members, but has "..(tmp and "true" or "nil"))
  assert(HSK:GetPresentListSize() == 0, "The Present list still be empty, but has non-zero size")
  assert(1 == HSK:GetMasterPosition(Test.Mains[1]), Test.Mains[1].." should still be in position 1")
  assert(2 == HSK:GetMasterPosition(Test.Mains[2]), Test.Mains[2].." should still be in position 2")
  tmpN = HSK:GetRoleCount(Test.Roles[1])
  assert(tmpN == 1, "Should have 1 member with role \""..Test.Roles[1].."\" but have "..(tmpN or "nil"))
  tmpN = HSK:GetRoleCount(Test.Roles[2])
  assert(tmpN == 1, "Should have 1 member with role \""..Test.Roles[2].."\" but have "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[1])
  assert(tmpN == 1, Test.Mains[1].." should have 1 toon, but has "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[2])
  assert(tmpN == 1, Test.Mains[2].." should have 1 toon, but has "..(tmpN or "nil"))

  tmp = HSK:AddMember(Test.Mains[3],Test.Roles[3],2)
  assert(2 == tmp, Test.Mains[3].." should now be at position 2, but is "..(tmp and "true" or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[1])
  assert(1 == tmpN, Test.Mains[1].." should still be in position 1, but is "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[2])
  assert(3 == tmpN, Test.Mains[2].." should now be in position 3, but is "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[1])
  assert(tmpN == 1, Test.Mains[1].." should have 1 toon, but has "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[2])
  assert(tmpN == 1, Test.Mains[2].." should have 1 toon, but has "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 1, Test.Mains[3].." should have 1 toon, but has "..(tmpN or "nil"))
  HSK:Print("*** AddMember() seems to work as expected.")

  --- AddAlt
  HSK:Print("*** Testing AddAlt()")
  tmp = HSK:AddAlt("nonexistent",""..Test.Alts[1].."")
  assert(nil == tmp, "AddAlt should have failed with a nonexistent main, but returned "..(tmp and "true" or "nil"))
  tmp = HSK:AddAlt(Test.Mains[1],Test.Alts[1])
  assert(1 == tmp, "Should have got position 1 for AddAlt, but got "..(tmp and "true" or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Alts[1])
  assert(1 == tmpN, Test.Alts[1].." was not in position 1, but "..(tmpN or "nil"))
  tmp = HSK:GetMain(Test.Alts[1])
  assert(Test.Mains[1] == tmp, Test.Mains[1].." should be the main of "..Test.Alts[1]..", but got: "..(tmp and "true" or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[1])
  assert(tmpN == 2, Test.Mains[1].." should now have 2 toons, but has "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[2])
  assert(tmpN == 1, Test.Mains[2].." should have 1 toon, but has "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 1, Test.Mains[3].." should have 1 toon, but has "..(tmpN or "nil"))
  tmp = HSK:AddAlt(Test.Mains[3],Test.Alts[3])
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 2, Test.Mains[3].." should have 2 toons, but has "..(tmpN or "nil"))
  tmp = HSK:AddAlt(Test.Mains[3],Test.Alts[3])
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 2, Test.Mains[3].." should still have 2 toons, but has "..(tmpN or "nil"))
  tmpN = HSK:GetRoleCount(Test.Roles[3])
  assert(tmpN == 2, "Should have 2 members with role \""..Test.Roles[3].."\" but has "..(tmpN or "nil"))
  tmp = HSK:AddAlt(Test.Mains[2],Test.Alts[3])
  assert(nil == tmp, "Should not have been able to add "..Test.Alts[3].." to "..Test.Mains[2]..", since it already exists on "..Test.Mains[3]..", got: "..(tmp and "true" or "nil"))
  HSK:Print("*** AddAlt() seems to work as expected.")

  --- RemoveMember
  HSK:Print("*** Testing RemoveMember()")
  tmp = HSK:RemoveMember(Test.Mains[3])
  assert(Test.Mains[3] == tmp, "Should have got main name to confirm, instead got: "..(tmp and "true" or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[3])
  assert(2 == tmpN, Test.Mains[3].." should still be at position 2, but is "..(tmpN or "nil"))
  tmpN = HSK:GetRoleCount(Test.Roles[3])
  assert(tmpN == 2, "Should still have 2 members with role \""..Test.Roles[3].."\" but has "..(tmpN or "nil"))
  tmp = HSK:RemoveMember(Test.Mains[3], true)
  assert(true == tmp, Test.Mains[3].." should have been removed, instead returned "..(tmp and "true" or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 0, Test.Mains[3].." should now have 0 toons, but has "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Alts[3])
  assert(tmpN == 0, Test.Alts[3].." should also have 0 toons, but has "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[3])
  assert(nil == tmpN, Test.Mains[3].." should have nil master position, not "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Alts[3])
  assert(nil == tmpN, Test.Alts[3].." should have nil master position, not "..(tmpN or "nil"))
  tmpN = HSK:GetRoleCount(Test.Roles[3])
  assert(tmpN == 0, "Should have 0 members with role \""..Test.Roles[3].."\" but has "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[1])
  assert(1 == tmpN, Test.Mains[1].." should still be in position 1, but is "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[2])
  assert(2 == tmpN, Test.Mains[2].." should now be back in position 2, but is "..(tmpN or "nil"))
  tmp = HSK:AddMember(Test.Mains[3],Test.Roles[3],2)

  tmp = HSK:RemoveMember(2)
  assert(Test.Mains[3] == tmp, "Should have got main name to confirm, instead got: "..(tmp and "true" or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[3])
  assert(2 == tmpN, Test.Mains[3].." should now be at position 2, but is "..(tmpN or "nil"))
  tmp = HSK:RemoveMember(2,true)
  assert(true == tmp, "Failed to remove 2nd member, "..Test.Mains[3])
  tmpN = HSK:GetMasterPosition(Test.Mains[3])
  assert(nil == tmpN, Test.Mains[3].." should have nil master position, not "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[1])
  assert(1 == tmpN, Test.Mains[1].." should still be in position 1, but is "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[2])
  assert(2 == tmpN, Test.Mains[2].." should now be back in position 2, but is "..(tmpN or "nil"))

  HSK:AddMember(Test.Mains[3],Test.Roles[3],2)
  HSK:AddAlt(Test.Mains[3],Test.Alts[3])
  tmp = HSK:RemoveMember(Test.Alts[3])
  assert(Test.Mains[3] == tmp, "Should have got main name to confirm, instead got: "..(tmp and "true" or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Alts[3])
  assert(2 == tmpN, Test.Alts[3].." should now be at position 2, but is "..(tmpN or "nil"))
  tmp = HSK:RemoveMember(Test.Alts[3], true)
  assert(true == tmp, Test.Mains[3].." should have been removed, instead returned "..(tmp and "true" or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[3])
  assert(nil == tmpN, Test.Mains[3].." should have nil master position, not "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[1])
  assert(1 == tmpN, Test.Mains[1].." should still be in position 1, but is "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[2])
  assert(2 == tmpN, Test.Mains[2].." should now be back in position 2, but is "..(tmpN or "nil"))
  HSK:Print("*** RemoveMember() seems to work as expected.")

  --- RemoveAlt
  HSK:Print("*** Testing RemoveAlt()")
  HSK:AddMember(Test.Mains[3],Test.Roles[3],2)
  HSK:AddAlt(Test.Mains[3],Test.Alts[3])
  HSK:AddAlt(Test.Mains[3],Test.Alts[4])
  HSK:AddAlt(Test.Mains[3],Test.Alts[5])
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 4, Test.Mains[3].." should have 4 toons before removal, but has "..(tmpN or "nil"))
  tmp = HSK:RemoveAlt(Test.Mains[3]) -- false
  assert(false == tmp, Test.Mains[3].." should have returned false: it's a main, instead got "..(tmp and "true" or "nil"))
  tmp = HSK:RemoveAlt("nosuchalt") -- nil
  assert(nil == tmp, "'nosuchalt' should have returned nil: it doesn't exist; instead got "..(tmp and "true" or "nil"))
  tmp = HSK:RemoveAlt(Test.Alts[3]) -- true
  assert(true == tmp, "Should have return 'true', instead got "..(tmp and "true" or "nil"))
  tmp = HSK:RemoveAlt(Test.Alts[3]) -- nil
  assert(nil == tmp, Test.Alts[3].." should have returned nil: it doesn't exist; instead got "..(tmp and "true" or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[3])
  assert(2 == tmpN, Test.Mains[3].." should still be in position 2, but is "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 3, Test.Mains[3].." should have 3 toons, but has "..(tmpN or "nil"))
  HSK:Print("*** RemoveAlt() seems to work as expected.")

  --- PromoteAlt
  HSK:Print("*** Testing PromoteAlt()")
  HSK:AddAlt(Test.Mains[3],Test.Alts[3])
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 4, Test.Mains[3].." should have 4 toons, but has "..(tmpN or "nil"))
  tmpN = HSK:PromoteAlt()
  assert(nil == tmpN, "Promoting nothing should fail, but returned "..(tmpN or "nil"))
  tmpN = HSK:PromoteAlt(Test.Mains[3])
  assert(2 == tmpN, "Promoting "..Test.Mains[3].." should return position 2, but returned "..(tmpN or "nil"))
  HSK:SetDropOnMainChange(false)
  tmpN = HSK:PromoteAlt(Test.Alts[4])
  assert(tmpN == HSK:GetMasterListSize(), "Promoting "..Test.Alts[4].." should return position 2, but returned "..(tmpN or "nil"))
  tmpN = HSK:PromoteAlt(Test.Mains[3])
  HSK:SetDropOnMainChange(true)
  assert(tmpN == HSK:GetMasterListSize(), "Promoting "..Test.Mains[3].." should return position "..HSK:GetMasterListSize()..", but returned "..(tmpN or "nil"))
  HSK:Print("*** PromoteAlt() seems to work as expected")

  --- Clear
  HSK:Print("*** Testing Clear()")
  assert(HSK:Clear(), "Clear failed. WTF?")
  tmpN = HSK:GetMasterListSize()
  assert(0 == tmpN, "The Master list should still be empty, but is "..(tmpN or "nil"))
  tmpN = HSK:GetPresentListSize()
  assert(0 == tmpN, "The Present list should still be empty, but is "..(tmpN or "nil"))
  HSK:Print("*** Clear() seems to work as expected")

  --- Bulk add
  HSK:Print("*** Testing SetMasterList()")
  tmpN = HSK:SetMasterList(Test.BulkMains)
  assert(10 == tmpN, "Should have 10 new members, instead have "..(tmpN or "nil"))
  tmpN = HSK:GetRoleCount("R1")
  assert(4  == tmpN, "Should have 4 members of R1, intead have "..(tmpN or "nil"))
  for i=1,10 do
    tmpN = HSK:GetMasterPosition("Main"..i)
    assert(i == tmpN, "Main"..i.." should be in position "..i..", instead it's in "..(tmpN or "nil"))
    tmp = HSK:GetRole("Main"..i) -- next assertion is ugly thanks to 1-based tables.
    assert("R"..((i-1)%3+1) == tmp, "Main"..i.." should have role R"..((i-1)%3+1)..", instead it's "..(tmp or "nil"))
  end
  HSK:Clear()
  tmpN = HSK:SetMasterList(Test.BulkMains, Test.BulkAlts)
  assert(10 == tmpN, "Should have 10 new members, instead have "..(tmpN or "nil"))
  tmpTable = HSK:GetAlts(Test.Mains[1])
  assert(nil ~= tmpTable, "tmpTable is nil. It shouldn't be.")
  assert(2 == #tmpTable, Test.Mains[1].." should have 2 alts, but has "..(#tmpTable))
  tmpTable = HSK:GetAlts(Test.Mains[2])
  assert(0 == #tmpTable, Test.Mains[2].." should have 0 alts, but has "..(#tmpTable))
  tmpTable = HSK:GetAlts(Test.Mains[3])
  assert(3 == #tmpTable, Test.Mains[3].." should have 3 alts, but has "..(#tmpTable))
  tmpTable = HSK:GetAlts(Test.Mains[4])
  assert(0 == #tmpTable, Test.Mains[4].." should have 0 alts, but has "..(#tmpTable))
  self:Print(HSK:ExportMasterList())
  -- HSK:Print("*** SetMasterList() seems to work as expected")

  HSK:Print("*!* Tests completed successfully *!* ")
end

-------- TEST DATA -------
Test = {}
Test.Roles   = {"R1","R2","R3",}
Test.Mains   = {"Main1","Main2","Main3","Main4","Main5","Main6","Main7","Main8","Main9","Main10",}
Test.Alts    = {"Main1Alt1","Main1Alt2","Main3Alt1","Main3Alt2","Main3Alt3",}
Test.BulkMains = {"Main1","R1","Main2","R2","Main3","R3","Main4","R1","Main5","R2","Main6","R3","Main7","R1","Main8","R2","Main9","R3","Main10","R1",}
Test.BulkAlts  = {["Main1"]={"Main1Alt1","Main1Alt2",},["Main3"]={"Main3Alt1","Main3Alt2","Main3Alt3",},}
Test.BadRole = "R0"
Test.BadMain = "Main0"
Test.BadAlt  = "Main0Alt0"
-------- MAIN -------
HSK:Test()
